@@ -2,7 +2,7 @@ package DateTime::Format::Flexible;
 use strict;
 use warnings;
 
-our $VERSION = '0.26';
+our $VERSION = '0.27';
 
 use base 'DateTime::Format::Builder';
 
@@ -14,19 +14,20 @@ use Carp 'croak';
 my $DELIM  = qr{(?:\\|\/|-|\.|\s)};
 my $HMSDELIM = qr{(?:\.|:)};
 my $YEAR = qr{(\d{1,4})};
-my $MON = qr{(\d\d?)};
-my $DAY = qr{(\d\d?)};
-my $HOUR = qr{(\d\d?)};
-my $HM = qr{(\d\d?)$HMSDELIM(\d\d?)};
-my $HMS = qr{(\d\d?)$HMSDELIM(\d\d?)$HMSDELIM(\d\d?)};
-my $HMSNS = qr{T?(\d\d?)$HMSDELIM(\d\d?)$HMSDELIM(\d\d?)$HMSDELIM(\d+)T?};
+my $MON = qr{([0-1]?\d)};
+my $DAY = qr{([0-3]?\d)};
+my $HOUR = qr{([0-2]?\d)};
+my $HM = qr{([0-2]?\d)$HMSDELIM([0-5]?\d)};
+my $HMS = qr{([0-2]?\d)$HMSDELIM([0-5]?\d)$HMSDELIM([0-5]?\d)};
+my $HMSNS = qr{T?([0-2]?\d)$HMSDELIM([0-5]?\d)$HMSDELIM([0-5]?\d)$HMSDELIM(\d+)T?};
 my $AMPM = qr{(a\.?m?|p\.?m?)\.?}i;
 
-my $MMDDYYYY = qr{(\d{1,2})$DELIM(\d{1,2})$DELIM(\d{1,4})};
-my $YYYYMMDD = qr{(\d{4})$DELIM(\d{1,2})$DELIM(\d{1,2})};
+my $MMDDYYYY = qr{(\d{1,2})${DELIM}(\d{1,2})${DELIM}(\d{1,4})};
+my $YYYYMMDD = qr{(\d{4})${DELIM}${MON}${DELIM}${DAY}};
+my $YYMMDD = qr{(\d\d)${DELIM}?([0-1]\d)${DELIM}?([0-3]\d)};
 my $MMYY = qr{(\d{1,2})${DELIM}(\d{1,2})}; # YEAR must be > 31 unless MMYY
 my $MMDD = qr{(\d{1,2})$DELIM(\d{1,2})};
-my $XMMXDD = qr{X(\d{1,2})X${DELIM}?(\d{1,2})};
+my $XMMXDD = qr{X([0-1]?\d)X${DELIM}?([0-3]?\d)};
 my $DDXMMX = qr{(\d{1,2})${DELIM}?X(\d{1,2})X};
 my $DDXMMXYYYY = qr{(\d{1,2})${DELIM}X(\d{1,2})X$DELIM(\d{1,4})};
 my $MMYYYY = qr{(\d{1,2})$DELIM(\d{4})};
@@ -65,6 +66,8 @@ my $MDYHMS = [ qw( month day year hour minute second ) ];
 my $MDYHMAP = [ qw( month day year hour minute ampm ) ];
 my $MDYHMSAP = [ qw( month day year hour minute second ampm ) ];
 my $MDHMSY = [ qw( month day hour minute second year ) ];
+my $MDHMSNSY = [ qw( month day hour minute second nanosecond year ) ];
+
 
 my $Y = [ qw( year ) ];
 my $YM = [ qw( year month ) ];
@@ -96,7 +99,6 @@ sub base
 my $formats =
 [
  [ preprocess => \&_fix_alpha ] ,
-
  { length => [18..22] , params => $YMDHMSAP , regex => qr{\A(\d{4})$DELIM(\d{2})$DELIM(\d{2})\s$HMS\s?$AMPM\z} , postprocess => \&_fix_ampm } ,
 
  # 2011-06-16-17.43.30.000000
@@ -107,19 +109,17 @@ my $formats =
  # M/DD/Y, MM/D/Y, M/D/Y, MM/DD/Y, M/D/YY, M/DD/YY, MM/D/Y, MM/SS/YY,
  # M/D/YYYY, M/DD/YYYY, MM/D/YYYY, MM/DD/YYYY
 
- { length => [5..10],  params => $MDY,      regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\z},               postprocess => \&_fix_year },
+ { length => [5..10],  params => $MDY,      regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\z},               postprocess => [ \&_fix_year, \&_fix_zero_month ] },
  { length => [12..14], params => $MDY,      regex => qr{\AX${MON}X${DELIM}n${DAY}n${DELIM}${YEAR}\z} },
- { length => [11..19], params => $MDYHMS,   regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HMS\z},         postprocess => \&_fix_year },
- { length => [11..20], params => $MDYHMAP,  regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HM\s?$AMPM\z},  postprocess => [ \&_fix_ampm , \&_fix_year ] } ,
- { length => [14..22], params => $MDYHMSAP, regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HMS\s?$AMPM\z}, postprocess => [ \&_fix_ampm , \&_fix_year ] } ,
+ { length => [11..19], params => $MDYHMS,   regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HMS\z},         postprocess => [ \&_fix_year, \&_fix_zero_month ] },
+ { length => [11..20], params => $MDYHMAP,  regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HM\s?$AMPM\z},  postprocess => [ \&_fix_ampm , \&_fix_year, \&_fix_zero_month ] } ,
+ { length => [14..22], params => $MDYHMSAP, regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HMS\s?$AMPM\z}, postprocess => [ \&_fix_ampm , \&_fix_year, \&_fix_zero_month ] } ,
 
  # 02/28/2014 14:30 (missing seconds)
- { length => [14..16], params => $MDYHM,    regex => qr{\A$MMDDYYYY\s$HM\z}, postprocess => \&_set_default_seconds } ,
+ { length => [14..16], params => $MDYHM,    regex => qr{\A$MMDDYYYY\s$HM\z}, postprocess => [\&_set_default_seconds, \&_fix_zero_month] } ,
 
  ########################################################
  ##### Year/Month/Day
- ##### Can't have 1,2 digit years in this format, would get confused
- ##### with MM-DD-YY
  # YYYY/M/D, YYYY/M/DD, YYYY/MM/D, YYYY/MM/DD
  # YYYY/MM/DD HH:MM:SS
  # YYYY-MM HH:MM:SS
@@ -133,6 +133,10 @@ my $formats =
  { length => [14..19], params => $YMDHMAP,  regex => qr{\A$YYYYMMDD\s$HM\s?$AMPM\z}, postprocess => \&_fix_ampm },
  { length => [14..19], params => $YMDHMS,   regex => qr{\A$YYYYMMDD\s$HMS\z} },
  { length => [17..21], params => $YMDHMSAP, regex => qr{\A$YYYYMMDD\s$HMS\s?$AMPM\z}, postprocess => \&_fix_ampm },
+ # 950404 00:22:12 => 1995-04-04T00:22:12
+ { length => [15],     params => $YMDHMS,   regex => qr{\A$YYMMDD\s$HMS\z}, postprocess => \&_fix_year },
+ { length => [19],     params => $YMDHMSNS, regex => qr{\A$YYMMDD\s$HMSNS\z}, postprocess => \&_fix_year },
+
 
  ########################################################
  ##### YYYY-MM-DDTHH:MM:SS
@@ -165,7 +169,7 @@ my $formats =
  ##### year must be 4 digits unless it is > 31
  ##### or MMYY is true
  # M/YYYY, MM/YYYY
- { length => [6,7], params => $MY, regex => qr{\A$MMYYYY\z} },
+ { length => [6,7], params => $MY, regex => qr{\A$MMYYYY\z}, postprocess => \&_fix_zero_month },
  { length => [3..5], params => $MY, regex => qr{\A$MMYY\z},
    postprocess => [sub {
        my %args = @_;
@@ -188,20 +192,20 @@ my $formats =
  { length => [12..17], params => $MDHMSAP, regex => qr{\A$MMDD\s$HMS\s?$AMPM\z},
    postprocess => [sub { my %args = @_; $args{parsed}{year} = __PACKAGE__->base->year },\&_fix_ampm] },
 
-
  ########################################################
  ##### Dates with at in their name: 12-10-65 at 5:30:25
  # the language plugins should wrap the time like this: T5:30:25T
  # 2005-06-12 T3Tp (15)
- { length => [16],     params => $MDYHMS,   regex => qr{\A${MON}${DAY}${YEAR}T${HMS}T\z},                               postprocess => \&_fix_year } ,
- { length => [17,18],  params => $MDYHMS,   regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s?T${HMS}T\z},            postprocess => \&_fix_year } ,
+ { length => [16,17],    params => $MDYHMS,   regex => qr{\A${MON}${DAY}${YEAR}T${HMS}T\z},                               postprocess => [ \&_fix_year, \&_fix_zero_month ] } ,
+ { length => [17,18,20], params => $MDYHMS,   regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s?T${HMS}T\z},            postprocess => [ \&_fix_year, \&_fix_zero_month ] } ,
  { length => [20],     params => $MDYHMS,   regex => qr{\AX${MON}X${DELIM}${DAY}${DELIM}${YEAR}\s?T${HMS}T\z},          postprocess => \&_fix_year } ,
+ { length => [21],     params => $MDYHMAP,  regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s?T${HM}T\s${AMPM}\z},    postprocess => [ \&_fix_ampm, \&_fix_zero_month ] } ,
  { length => [20,21],  params => $YMDHMAP,  regex => qr{\A${YYYYMMDD}\s?T${HM}T\s${AMPM}\z},                            postprocess => \&_fix_ampm } ,
  { length => [21,22],  params => $YMDHMSAP, regex => qr{\A${YEAR}${MON}${DAY}\s?T${HMS}T\s${AMPM}\z},                   postprocess => \&_fix_ampm } ,
  { length => [15],     params => $YMDHAP,   regex => qr{\A${YEAR}${DELIM}${MON}${DELIM}${DAY}\s?T${HOUR}T\s?${AMPM}\z}, postprocess => \&_fix_ampm } ,
  { length => [16..18], params => $YMDHM,    regex => qr{\A${YEAR}${DELIM}${MON}${DELIM}${DAY}\s?T${HM}T\z},             postprocess => \&_fix_year } ,
  { length => [21],     params => $YMDHMS,   regex => qr{\A${YEAR}${DELIM}${MON}${DELIM}${DAY}\s?T${HMS}T\z},            postprocess => \&_fix_year } ,
- { length => [16],     params => $MDYHMS,   regex => qr{\A${MON}${DAY}(\d\d)\s?T${HMS}T\z},                             postprocess => \&_fix_year } ,
+ { length => [16],     params => $MDYHMS,   regex => qr{\A${MON}${DAY}(\d\d)\s?T${HMS}T\z},                             postprocess => [ \&_fix_year, \&_fix_zero_month ] } ,
  { length => [16],     params => $YMDHAP,   regex => qr{\A${YEAR}${DELIM}${MON}${DELIM}${DAY}\s?T${HOUR}T${AMPM}\z},    postprocess => \&_fix_ampm } ,
 
  { length => [15,16],  params => $MDHMS,    regex => qr{\A${MON}${DELIM}${DAY}\s?T${HMS}T\z},
@@ -252,6 +256,10 @@ my $formats =
  { length => [16..25], params => $DMYHMSNS, regex => qr{\A${DDXMMXYYYY}\s${HMSNS}\z},       postprocess => \&_fix_year },
  { length => [14..24], params => $DMYHMSAP, regex => qr{\A${DDXMMXYYYY}\s${HMS}\s?$AMPM\z}, postprocess => [ \&_fix_year , \&_fix_ampm ] },
  { length => [9..15] , params => $HMSMD,    regex => qr{\A${HMS}${XMMXDD}\z},               postprocess => \&_set_default_year },
+
+ # Fri Dec  2 22:56:03.500 GMT+0 1994
+ { length => [24],     params => $MDHMSNSY, regex => qr{\A${XMMXDD}\s${HMSNS}\s${YEAR}\z}, },
+
  { length => [9..15] , params => $HMSDM,    regex => qr{\A${HMS}${DELIM}?${DDXMMX}\z},      postprocess => \&_set_default_year },
  { length => [11..17], params => $HMSMDY,   regex => qr{\A${HMS}${XMMXDDYYYY}\z},           postprocess => \&_fix_year },
  { length => [6..11],  params => $HMMD,     regex => qr{\A${HM}${XMMXDD}\z},                postprocess => \&_set_default_year },
@@ -554,8 +562,9 @@ sub _parse_timezone
     # must be surrounded by spaces
     # 5:30 pm GMT 121065
     # Tue Feb 28 14:30:00 UTC 2014
+    # Fri Dec  2 22:56:03 GMT+0 1994
 
-    if ( my ( $tz ) = $date =~ m{\s(GMT|UTC)\s}mx )
+    if ( my ( $tz ) = $date =~ m{\s(GMT(?:\+0)?|UTC)\s}mx )
     {
         $date =~ s{\Q$tz\E}{};
         $p->{time_zone} = 'UTC';
@@ -634,6 +643,7 @@ sub _clean_whitespace
     $string =~ s{\s+\z}{}mx;    # trim back
 
     $string =~ s{\s+}{ }gmx;    # remove extra whitespace from the middle
+    $string =~ s{"}{}gmx;
     return $string;
 }
 
@@ -670,6 +680,23 @@ sub _fix_ampm
     return 1;
 }
 
+sub _fix_zero_month
+{
+    my %args = @_;
+
+    return 1 if not exists $args{parsed}{month};
+    return 1 if not defined $args{parsed}{month};
+
+    if ($args{parsed}{month} == 0)
+    {
+        # they probably meant october
+        print( "#  month => 0 => 10\n") if $ENV{DFF_DEBUG};
+        $args{parsed}{month} = 10;
+    }
+
+    return 1;
+}
+
 sub _set_default_seconds
 {
     my %args = @_;
